#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#define MAX_N        16
#define MAX_NODES   341
#define INPUT "D.txt"

FILE *ifp;

typedef struct {
  int intensity;             /* -1 if non-leaf */
  int child[4];
} Node;

int n;
int image1[MAX_N][MAX_N], image2[MAX_N][MAX_N];
Node tree1[MAX_NODES], tree2[MAX_NODES];

void read_tree(Node *tree)
{
  int leafs, num_nodes, i, k;

  num_nodes = 0;
  for (i = 1; i <= n; i *= 2) {
    num_nodes += i*i;
  }

  for (i = 0; i < num_nodes/4; i++) {
    tree[i].intensity = -1;
    for (k = 0; k < 4; k++) {
      tree[i].child[k] = 4*i+k+1;
    }
  }

  fscanf(ifp, "%d", &leafs);
  for (i = 0; i < leafs; i++) {
    fscanf(ifp, "%d", &k);
    fscanf(ifp, "%d", &(tree[k].intensity));
  }
}

void decode_image(Node *tree, int image[MAX_N][MAX_N], int row, int col, 
		  int size, int root, int scramble)
{
  int i, j, r, c;

  assert(size > 0);
  
  if (tree[root].intensity == -1) {
    /* internal node */
    decode_image(tree, image, row, col, size/2, 
		 tree[root].child[0], scramble);
    decode_image(tree, image, row, col+size/2, size/2, 
		 tree[root].child[1], scramble);
    decode_image(tree, image, row+size/2, col, size/2, 
		 tree[root].child[2], scramble);
    decode_image(tree, image, row+size/2, col+size/2, size/2, 
		 tree[root].child[3], scramble);
  } else {
    /* leaf */
    for (i = row; i < row+size; i++) {
      for (j = col; j < col+size; j++) {
	if (!scramble) {
	  image[i][j] = tree[root].intensity;
	} else {
	  r = image1[i][j] / n;
	  c = image1[i][j] % n;
	  image[r][c] = tree[root].intensity;
	}
      }
    }
  }
}


int main(void)
{
  int cases, k, i, j;

  if ( (ifp=fopen(INPUT,"r")) == NULL)
  	  {printf("Cannot open %s file.\n",INPUT); exit;}

  fscanf(ifp, "%d", &cases);
  for (k = 0; k < cases; k++) {
    fscanf(ifp, "%d", &n);
    
    read_tree(tree1);
    read_tree(tree2);

    decode_image(tree1, image1, 0, 0, n, 0, 0);
    decode_image(tree2, image2, 0, 0, n, 0, 1);

    if (k) {
      printf("\n");
    }
    printf("Case %d\n\n", k+1);

    for (i = 0; i < n; i++) {
      for (j = 0; j < n; j++) {
	printf("%4d", image2[i][j]);
      }
      printf("\n");
    }
  }

  return 0;
}
